/*
 Copyright (c) 2006-2016 Trellis Management Co. Ltd.

 This software module is an implementation of one or more tools as proposed
 for the JPEG XT standard.
 
 The copyright in this software is being made available under the
 license included with the sources LICENSE.TXT. This software may be subject to other third
 party and contributor rights, including patent rights, and no such
 rights are granted under this license. 
*/

#ifndef _image_h
#define _image_h

// Image structure
typedef struct _image {
    unsigned char *data;
    unsigned int data_size;
    unsigned width;
    unsigned height;
    unsigned format;
    int type;
    int bpp;
    int grayscale;
} FILE_IMAGE;

typedef struct _image_pixel_float {
    float red;
    float green;
    float blue;
} IMAGE_PIXELF;

typedef struct _image_pixel_float_bw {
    float gray;
} IMAGE_PIXELF_BW;


typedef struct _mem_image {
    unsigned char *data;
    unsigned int data_size;
    unsigned width;
    unsigned height;
    int bpp;
    int type;
    int grayscale;
} MEMORY_FILE_IMAGE;

typedef struct vector {
    float x;
    float y;
    float z;
} COLOR_VECTOR;

// Image types
enum {
    IMAGE_TYPE_JPEG,
    IMAGE_TYPE_HDR,
    IMAGE_TYPE_ZIP,
    IMAGE_TYPE_BMP,
    IMAGE_TYPE_RGB16,
    IMAGE_TYPE_RGB32
};

float clamp(float value,float maxf,float minf);
FILE_IMAGE *allocate_image (int width, int height, int bpp, int channels);
void free_image(FILE_IMAGE *image); // Free previously loaded image
void free_memory_image ( MEMORY_FILE_IMAGE *image );
int flip_horizontal( FILE_IMAGE *image );
int flip_vertical( FILE_IMAGE *image );
unsigned char *get_scanline(FILE_IMAGE *image,unsigned scanline);
char *get_extension(const char *filename);
FILE_IMAGE *image_convert_float(FILE_IMAGE *src);
FILE_IMAGE *image_convert_24(FILE_IMAGE *src);
FILE_IMAGE *image_convert_rgb_xyz(FILE_IMAGE *src, float matrix[]);
FILE_IMAGE *image_convert_xyz_rgb(FILE_IMAGE *src, float matrix[]);
int image_linearize(FILE_IMAGE *image, float gamma);
int linear_to_srgb(FILE_IMAGE *image);
int srgb_to_linear(FILE_IMAGE *image);
FILE_IMAGE *image_copy(FILE_IMAGE *source);
FILE_IMAGE *toRgb(FILE_IMAGE *rgba_image);
void image_minmax_values(FILE_IMAGE *image,float *min_r,float *min_g,float *min_b,float *max_r,float *max_g,float *max_b);
void shift_ldr_image(FILE_IMAGE *image);
MEMORY_FILE_IMAGE *file_to_mem(FILE_IMAGE *image);
FILE_IMAGE *rgbTogray(FILE_IMAGE *rgb_image);
#endif
