/*
 Copyright (c) 2006-2016 Trellis Management Co. Ltd.

 This software module is an implementation of one or more tools as proposed
 for the JPEG XT standard.
 
 The copyright in this software is being made available under the
 license included with the sources LICENSE.TXT. This software may be subject to other third
 party and contributor rights, including patent rights, and no such
 rights are granted under this license. 
*/
#ifndef DONT_USE_PNG
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "png_io.h"


FILE_IMAGE *png_load(const char *filename)
{
    png_byte header[8];
    FILE_IMAGE *out_image;
    FILE_IMAGE *out_alpha;
    unsigned int i;
    png_bytep * row_pointers;
	png_structp png_ptr;
	png_infop info_ptr;
	png_infop end_info;
	int bit_depth, color_type;
	png_uint_32 temp_width, temp_height;
	int rowbytes;
	int channels;
	png_byte *image_data=NULL;

    FILE *fp = fopen(filename, "rb");
    if (fp == 0)
    {
        return NULL;
    }
    fread(header, 1, 8, fp);
    if (png_sig_cmp(header, 0, 8))
    {
        fclose(fp);
        return NULL;
    }
    png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
    if (!png_ptr)
    {
        fclose(fp);
        return NULL;
    }
    info_ptr = png_create_info_struct(png_ptr);
    if (!info_ptr)
    {
        png_destroy_read_struct(&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
        fclose(fp);
        return NULL;
    }
    end_info = png_create_info_struct(png_ptr);
    if (!end_info)
    {
        png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp) NULL);
        fclose(fp);
        return NULL;
    }
    // the code in this if statement gets called if libpng encounters an error
    if (setjmp(png_jmpbuf(png_ptr))) {
        png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
        fclose(fp);
        return NULL;
    }

    // init png reading
    png_init_io(png_ptr, fp);

    // let libpng know you already read the first 8 bytes
    png_set_sig_bytes(png_ptr, 8);

    // read all the info up to the image data
    png_read_info(png_ptr, info_ptr);

    // variables to pass to get info
    
    
    // get info about png
    png_get_IHDR(png_ptr, info_ptr, &temp_width, &temp_height, &bit_depth, &color_type,
        NULL, NULL, NULL);
    if (! (color_type & PNG_COLOR_MASK_COLOR))
    {
        png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
        fclose(fp);
        return NULL;
    }
    // Update the png info struct.
    png_read_update_info(png_ptr, info_ptr);

    // Row size in bytes.
    rowbytes = png_get_rowbytes(png_ptr, info_ptr);

    channels=png_get_channels(png_ptr, info_ptr);
    //if (channels>3)
    //{
    //    png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
    //    fclose(fp);
    //    return NULL;
   // }
    
    if ((image_data = (unsigned char *)malloc(rowbytes*temp_height)) == NULL) {
            png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
            return NULL;
        }
    if (channels==3)
        out_image = allocate_image(temp_width,temp_height,24);
    else
        out_image = allocate_image(temp_width,temp_height,32);

    // row_pointers is for pointing to image_data for reading the png with libpng
    row_pointers = malloc(temp_height * sizeof(png_bytep));
    if (row_pointers == NULL)
    {
        png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
        free(image_data);
        fclose(fp);
        return NULL;
    }

    for (i = 0; i < temp_height; i++)
    {
        row_pointers[temp_height - 1 - i] = image_data + (i * rowbytes);
    }

    // read the png into image_data through row_pointers
    png_read_image(png_ptr,row_pointers);
    png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
    free(row_pointers);
    fclose(fp);

    free(out_image->data);
    out_image->data=image_data;
    if (channels==4)
    {
        out_alpha=toRgb(out_image);
        free_image(out_image);
        return out_alpha;
    }
/*
    unsigned char *img=image_data;
    unsigned char *img2=out_image->data;
    for (int y=0;y<temp_height;y++)
    {
        unsigned char *p=img;
        unsigned char *p2=img2;
        unsigned off=0;
        unsigned pngoff=0;
        for (int x=0;x<temp_width;x++)
        {
            out_image->data[off]=image_data[pngoff];
            out_image->data[off+1]=image_data[pngoff+1];
            out_image->data[off+2]=image_data[pngoff+2];
            off+=3;
            pngoff+=3;
        }
        img+=rowbytes;
        img2+= (out_image->width*3);
    }
    */
    return out_image;
}
#endif
