/*
 Copyright (c) 2006-2016 Trellis Management Co. Ltd.

 This software module is an implementation of one or more tools as proposed
 for the JPEG XT standard.
 
 The copyright in this software is being made available under the
 license included with the sources LICENSE.TXT. This software may be subject to other third
 party and contributor rights, including patent rights, and no such
 rights are granted under this license. 
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "ppm_io.h"



int ppm_get_int(FILE *io) {
    char c = 0;
	int i = 0;
    int firstchar;
    
    // skip forward to start of next number
    fread(&c,1,1,io);
    while (1) {
        // eat comments
        
        if (c == '#') {
			// if we're at a comment, read to end of line
            
            firstchar = 1;
            
            while (1) {
                fread(&c,1,1,io);

                if (firstchar==1 && c == ' ') {
					// loop off 1 sp after #
                    
                    firstchar = 0;
				} else if (c == '\n') {
					break;
				}
			}
		}
        
        if (c >= '0' && c <='9') {
			// we've found what we were looking for
            
            break;
		}
        fread(&c,1,1,io);
    }
    
    // we're at the start of a number, continue until we hit a non-number
    
    
    
    while (1) {
        i = (i * 10) + (c - '0');
        
        fread(&c,1,1,io);
        
        if (c < '0' || c > '9')
            break;
    }
    
    return i;
}



FILE_IMAGE *ppm_load(const char *filename)
{
    int width;
    int height;
    int bits;
    int y;
    FILE_IMAGE *img=NULL;
    unsigned char *pimg;
    int lineWidth;
    unsigned char *lineBuffer;
    char fileType[3]={0};
	char dummy;
    unsigned input_channels=3;
    FILE *f=fopen(filename,"rb");
    if (f==NULL)
        return NULL;


    fscanf(f, "%s\n", fileType);
    if (strcmp(fileType,"P6")!=0)
    {
        if (strcmp(fileType,"P5")!=0)
        {
            fclose(f);
            return NULL;
        }
        else
            input_channels=1;
    }
    fscanf(f,"%i\n",&width);
    fscanf(f,"%i\n",&height);
    fscanf(f,"%i",&bits);
    
    fread(&dummy,1,1,f);

    img = allocate_image(width,height,24,input_channels);
    pimg = (unsigned char*)img->data;

    lineWidth = input_channels*width;
    lineBuffer=(unsigned char *)malloc(lineWidth);
    for (y = 0; y < height; y++) {
        unsigned char *oimg=(unsigned char *)pimg;
        fread(lineBuffer,1,lineWidth,f);
        memcpy(oimg,lineBuffer,lineWidth);
        pimg+=lineWidth;
    }
    free(lineBuffer);
    fclose(f);

    return img;
}

