/*
 Copyright (c) 2006-2016 Trellis Management Co. Ltd.

 This software module is an implementation of one or more tools as proposed
 for the JPEG XT standard.
 
 The copyright in this software is being made available under the
 license included with the sources LICENSE.TXT. This software may be subject to other third
 party and contributor rights, including patent rights, and no such
 rights are granted under this license. 
*/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "image.h"
#include "decoder.h"
#include "jpeg_io.h"
#include "pfm_io.h"
#include "common.h"
#include "checksum.h"


extern int g_Rw;
extern int g_log_decode;
// Reconstruct original float image
int xdp_HDRDecode(FILE_IMAGE *image_24bit,FILE_IMAGE *image_hdr,FILE_IMAGE **out_hdr_image,XDEPTH_INFO *xd_info)
{
    unsigned int h;
    unsigned int hx;
    unsigned char *data_jpg1;
    unsigned char *data_jpg2;
    unsigned int img_width,img_height;
    unsigned int pitch;
    unsigned input_channel=3;
    unsigned char *pimg1,*pimg2;
    float *img_float_buffer=NULL;
    unsigned int hdroff=0;
	FILE_IMAGE *dib_hdr;
	float *bitsRGB;
    float exposure_value=1.0f;

    if (image_24bit==NULL)
        return 0;
    if (image_hdr==NULL)
        return 0;

    data_jpg1=image_24bit->data;
    data_jpg2=image_hdr->data;
    img_width=image_24bit->width;
    img_height=image_24bit->height;
    if (image_24bit->grayscale==1)
        input_channel=1;
    pitch=img_width*input_channel;
    
    img_float_buffer=(float *)malloc(((img_width*img_height)*input_channel)*sizeof(float) );
    if (img_float_buffer==NULL)
        return 0;

    if (xd_info->expval!=0)
        exposure_value=xd_info->expval;
    pimg1=data_jpg1;
    pimg2=data_jpg2;
    for ( h=0;h<img_height;h++)
    {
        unsigned char *ppimg1=pimg1;
        unsigned char *ppimg2=pimg2;
        for ( hx=0;hx<img_width*input_channel;hx+=input_channel)
        {

            float t2_r,t2_g,t2_b;
            float color_jpg_1_g;
            float color_jpg_1_b;
            float color_jpg_2_g;
            float color_jpg_2_b;
            float color_jpg_1_r=toFloat(ppimg1[hdroff],g_Rw,xd_info->rd_e_param);
            float color_jpg_2_r=toFloat(ppimg2[hdroff],g_Rw,xd_info->rd_e_param);

            if (input_channel==3)
            {
                color_jpg_1_g=toFloat(ppimg1[hdroff+1],g_Rw,xd_info->rd_e_param);
                color_jpg_1_b=toFloat(ppimg1[hdroff+2],g_Rw,xd_info->rd_e_param);
                color_jpg_2_g=toFloat(ppimg2[hdroff+1],g_Rw,xd_info->rd_e_param);
                color_jpg_2_b=toFloat(ppimg2[hdroff+2],g_Rw,xd_info->rd_e_param);
            }

            if (xd_info->p1>0) // sRgb to linear
            {
                color_jpg_1_r  = color_jpg_1_r>=xd_info->p1?powf((color_jpg_1_r+xd_info->p3)/(1.0f+xd_info->p3),xd_info->p2):powf((xd_info->p1+xd_info->p3) / (1.0f+xd_info->p3),xd_info->p2) * color_jpg_1_r/xd_info->p1;
                 if (input_channel==3)
                 {
                    color_jpg_1_g= color_jpg_1_g>=xd_info->p1?powf((color_jpg_1_g+xd_info->p3)/(1.0f+xd_info->p3),xd_info->p2):powf((xd_info->p1+xd_info->p3) / (1.0f+xd_info->p3),xd_info->p2) * color_jpg_1_g/xd_info->p1;
                    color_jpg_1_b= color_jpg_1_b>=xd_info->p1?powf((color_jpg_1_b+xd_info->p3)/(1.0f+xd_info->p3),xd_info->p2):powf((xd_info->p1+xd_info->p3) / (1.0f+xd_info->p3),xd_info->p2) * color_jpg_1_b/xd_info->p1;
                 }
            }
            else // Inverse gamma correction
            {
                color_jpg_1_r=(float)pow((color_jpg_1_r+xd_info->p3)/(1.0f+xd_info->p3),xd_info->ldr_gamma);
                if (input_channel==3)
                {
                    color_jpg_1_g=(float)pow((color_jpg_1_g+xd_info->p3)/(1.0f+xd_info->p3),xd_info->ldr_gamma);
                    color_jpg_1_b=(float)pow((color_jpg_1_b+xd_info->p3)/(1.0f+xd_info->p3),xd_info->ldr_gamma);
                }
            }
            // Apply inverse HDR gamma value on residual data
            t2_r=(float)pow(color_jpg_2_r,xd_info->hdr_gamma);
            if (input_channel==3)
            {
                t2_g=(float)pow(color_jpg_2_g,xd_info->hdr_gamma);
                t2_b=(float)pow(color_jpg_2_b,xd_info->hdr_gamma);

                t2_g=(t2_g*(xd_info->max_g-xd_info->min_g))+xd_info->min_g;
                t2_b=(t2_b*(xd_info->max_b-xd_info->min_b))+xd_info->min_b;
            }

            // Scale redisual values at the original size
            t2_r=(t2_r*(xd_info->max_r-xd_info->min_r))+xd_info->min_r;

            if (input_channel==3)
            {
                // Check for XYZ color space
                if (xd_info->colorspace==CS_XYZ)
                {
                    float Xh,Yh,Zh;
                    // Convert RGB base into XYZ color space
                    float Xl = (xd_info->base_matrix[0] * color_jpg_1_r) + (xd_info->base_matrix[1] * color_jpg_1_g) + (xd_info->base_matrix[2] * color_jpg_1_b);
                    float Yl = (xd_info->base_matrix[3] * color_jpg_1_r) + (xd_info->base_matrix[4] * color_jpg_1_g) + (xd_info->base_matrix[5] * color_jpg_1_b);
                    float Zl = (xd_info->base_matrix[6] * color_jpg_1_r) + (xd_info->base_matrix[7] * color_jpg_1_g) + (xd_info->base_matrix[8] * color_jpg_1_b);
                    color_jpg_1_r = Xl;
                    color_jpg_1_g = Yl;
                    color_jpg_1_b = Zl;


                    // Check if we need to convert residial data as well
                    if (xd_info->resi_identity==0)
                    {
                        // Convert residual data into XYZ color space
                        Xh = (xd_info->resi_matrix[0] * t2_r) + (xd_info->resi_matrix[1] * t2_g) + (xd_info->resi_matrix[2] * t2_b);
                        Yh = (xd_info->resi_matrix[3] * t2_r) + (xd_info->resi_matrix[4] * t2_g) + (xd_info->resi_matrix[5] * t2_b);
                        Zh = (xd_info->resi_matrix[6] * t2_r) + (xd_info->resi_matrix[7] * t2_g) + (xd_info->resi_matrix[8] * t2_b);
                    }
                    else
                    {
                        // We don't need to convert residual into XYZ
                        Xh = t2_r;
                        Yh = t2_g;
                        Zh = t2_b;
                    }
                    t2_r = Xh;
                    t2_g = Yh;
                    t2_b = Zh;
                }
            }
            // Check if log decoding is requested
            if (xd_info->log_decoding==0)
            {
                // Decode image using division
                img_float_buffer[hdroff]=((float)color_jpg_1_r*((float)1.0/(t2_r+xd_info->epsilon_div)))*exposure_value;
                if (input_channel==3)
                {
                    img_float_buffer[hdroff+1]=((float)color_jpg_1_g*((float)1.0/(t2_g+xd_info->epsilon_div)))*exposure_value;
                    img_float_buffer[hdroff+2]=((float)color_jpg_1_b*((float)1.0/(t2_b+xd_info->epsilon_div)))*exposure_value;

                }
            }
            else
            {
                // Decode image use log operation
                //img = sign(P1) * log(pow(abs(P1)*rgb,P2)+P3)+P4
                float lR1=logf(color_jpg_1_r+xd_info->log_params_ldr[XT_P3]);
                float lG1;
                float lB1;
                float lG2;
                float lB2;
				float lR2;
                if (input_channel==3)
                {
                    lG1=logf(color_jpg_1_g+xd_info->log_params_ldr[XT_P3]);
                    lB1=logf(color_jpg_1_b+xd_info->log_params_ldr[XT_P3]);

                    lG2=COPYSIGN(1.0f,xd_info->log_params_hdr[XT_P1]) * logf(t2_g+xd_info->log_params_hdr[XT_P3]);
                    lB2=COPYSIGN(1.0f,xd_info->log_params_hdr[XT_P1]) * logf(t2_b+xd_info->log_params_hdr[XT_P3]);

                    img_float_buffer[hdroff+1]=exposure_value*exp((lG1+lG2));
                    img_float_buffer[hdroff+2]=exposure_value*exp((lB1+lB2));
                }
                lR2=COPYSIGN(1.0f,xd_info->log_params_hdr[XT_P1]) * logf(t2_r+xd_info->log_params_hdr[XT_P3]);
                // img = P3*exp(rgb*(P2-P1)+P1)+P4;
                img_float_buffer[hdroff]=exposure_value*exp((lR1+lR2));
            }
            input_channel==3?hdroff+=3:hdroff++;
        }
    }

    // Store float data in output buffer
    dib_hdr=allocate_image(img_width,img_height,96,input_channel);
    bitsRGB=(float*)dib_hdr->data;
    memcpy(bitsRGB,img_float_buffer,(sizeof(float)*(img_width*input_channel))*img_height);
    if (input_channel==1)
        dib_hdr->grayscale=1;

    *out_hdr_image=dib_hdr;
    
    free(img_float_buffer);
    return 1;
}


// Get two jpeg image LDR, HDR and process both with xdp_HDRDecode to reconstruct original float image
FILE_IMAGE *XDP_ExpandHDR(XDEPTH_INFO *params, FILE_IMAGE *ldr_image, MEMORY_FILE_IMAGE *hdr_image_mem)
{
    FILE_IMAGE *hdr_image;
    FILE_IMAGE *output_image=NULL;
	int ret;
    
    // Read hdr jpeg from memory


    hdr_image=jpeg_load_mem(hdr_image_mem);
    if (!hdr_image)
        return NULL;

    // Decode XDepth images into HDR float image.
    ret=xdp_HDRDecode(ldr_image,hdr_image,&output_image,params);
    if (!ret)
    {
        free_image(hdr_image);
        return NULL;
    }
    output_image->type=IMAGE_TYPE_HDR;

    return output_image;
}

// Decode XDepth File, open file and get XDepth header then call XDP_ExpandHDR
int xdp_decode(const char *filename,const char *out_filename)
{
    XDEPTH_INFO xdpchnk;
	int width;
    int height;
    int grayscale=0;
	FILE *inputFile;
    MEMORY_FILE_IMAGE ldr_image_mem;
    FILE_IMAGE *ldr_image;
	FILE_IMAGE *output_image;
	unsigned int fsize;
	unsigned short chksum;
    MEMORY_FILE_IMAGE *hdr_image_mem=NULL;
    unsigned char *tmp_p=NULL;
    unsigned short *jpeg_tags=NULL;
    int file_type_ok=0;
    int current_curv=0;
    int current_ftrx=0;
    JPXT_FP_LIN_TRAFO xt_lin_trafo[2];
    JPXT_CURV xtcurvs[15];
    unsigned short cur_len;
    int all_box_ok=0;
    unsigned int resi_off=0;
    int lpts_found=0,dpts_found=0,cpts_found=0,rpts_found=0,ocon_found=0;
    int lpts_td0,dpts_td0,cpts_td0,rpts_td0,ocon_to0;
    int lpts_td1,dpts_td1,cpts_td1,rpts_td1,ocon_to1;
    int lpts_td2,dpts_td2,cpts_td2,rpts_td2,ocon_to2;
	unsigned int lbox=0;
    int input_channels=3;
    if (grayscale==1)
        input_channels=1;
    jpeg_getSize(filename,&width,&height,&grayscale);
    inputFile=fopen(filename,"rb");
    if (inputFile==NULL)
        return XT_ERROR_NULL_POINTER;
    fseek(inputFile,0,SEEK_END);
    fsize=ftell(inputFile);
    fseek(inputFile,0,SEEK_SET);
    ldr_image_mem.data=(unsigned char *)malloc(fsize);
    if (ldr_image_mem.data==NULL)
    {
        fclose(inputFile);
        return XT_ERROR_NULL_POINTER;
    }
    fread(ldr_image_mem.data,fsize,1,inputFile);
    fclose(inputFile);
    ldr_image_mem.data_size=fsize;
    ldr_image_mem.bpp=24;
    ldr_image_mem.width=width;
    ldr_image_mem.height=height;
    ldr_image_mem.grayscale=grayscale;
    tmp_p =ldr_image_mem.data;
    jpeg_tags = (unsigned short *)ldr_image_mem.data;

    hdr_image_mem=(MEMORY_FILE_IMAGE*) malloc(sizeof(MEMORY_FILE_IMAGE));
    if (hdr_image_mem==NULL)
    {
        free(ldr_image_mem.data);
        return XT_ERROR_ALLOC;
    }
    hdr_image_mem->data=(unsigned char *)malloc(width*height*input_channels);
    hdr_image_mem->bpp=24;
    hdr_image_mem->width=width;
    hdr_image_mem->height=height;
    hdr_image_mem->grayscale=grayscale;

    memset(&xdpchnk,0,sizeof(XDEPTH_INFO));
    xdpchnk.resi_identity=1;
    xdpchnk.colorspace=CS_RGB;
    if (g_log_decode==1)
        xdpchnk.log_decoding=1;
    
    // Read all needed parameters to decode Profile B
    do {
        unsigned short cur_tag;
        XDP_SWAP_SHORT16(cur_tag,*jpeg_tags);
        if (cur_tag!=0xFFEB)
        {
            if (cur_tag==0xFFD8)
            {
                jpeg_tags++;
                tmp_p+=2;
            }
            else
            {

                jpeg_tags++;
                tmp_p+=2;
                XDP_SWAP_SHORT16(cur_len,*jpeg_tags);
                jpeg_tags=(unsigned short *)(tmp_p+cur_len);
                tmp_p=(unsigned char *)jpeg_tags;
                if (tmp_p-ldr_image_mem.data>=ldr_image_mem.data_size)
                    all_box_ok=1;
            }
        }
        else
        {
			unsigned char *payload;
			unsigned short payload_size;
            JPXT_SUPERBOX *xt_box_header=(JPXT_SUPERBOX*)tmp_p;
            int chsz;
            XDP_SWAP_SHORT32(lbox,xt_box_header->LBox);
            XDP_SWAP_SHORT16(chsz,xt_box_header->Le);
            XDP_SWAP_SHORT32(xt_box_header->LBox,xt_box_header->LBox);
            payload=(tmp_p+sizeof(JPXT_SUPERBOX));
            payload_size=xt_box_header->LBox-sizeof(xt_box_header->LBox)-sizeof(xt_box_header->TBox);
            tmp_p+=chsz+2;
            jpeg_tags=(unsigned short *)tmp_p;
            if (file_type_ok==0)
            {
                JPXT_FTYPE *xt_ftype;
                file_type_ok=1;
                // First Box expected if File Type Box
                if (memcmp((char *)&xt_box_header->TBox,"ftyp",4)!=0)
                {
                    // Jpeg XT error!
                    free(ldr_image_mem.data);
                    return XT_ERROR_INVALID_PROFILE;
                }
                xt_ftype = (JPXT_FTYPE*)payload;
                // Check if we are decoding Profile B
                if (memcmp((char *)&xt_ftype->CL,"xrxd",4)!=0)
                {
                    // Profile error
                    free(ldr_image_mem.data);
                    return XT_ERROR_INVALID_PROFILE;
                }
            }

            if (!memcmp((char *)&xt_box_header->TBox,"LCHK",4))
            {
                // Read Legacy Checksum and compare
                unsigned int *checksum=(unsigned int*)payload;
                XDP_SWAP_SHORT32(*checksum,*checksum);
                chksum=computeChecksum(&ldr_image_mem);
                if (*checksum!=chksum)
                {
                    free(ldr_image_mem.data);
                    return XT_ERROR_CHECKSUM;
                }
            }
            else if (!memcmp((char *)&xt_box_header->TBox,"SPEC",4))
            {
                unsigned int off=0;
                // Read Merging Specification Box Payload
                do {
                    JPXT_CURV *current_curv_box=(JPXT_CURV*)(payload+off);
                    XDP_SWAP_SHORT32(lbox,current_curv_box->LBox);
                    if (!memcmp(&current_curv_box->TBox,"CURV",4))
                    {
                        int M;
                        int T;
                        int e;
                        if (current_curv>=7 )
                        {
                            free(ldr_image_mem.data);
                            printf("Wrong jpeg-xt profile!\n");
                            return XT_ERROR_INVALID_INFO;
                        }

                        T=current_curv_box->Mt & 0x0f;
                        M=(current_curv_box->Mt & 0xf0)>>4;
                        e=(current_curv_box->et & 0xf0)>>4;
                        memcpy(&xtcurvs[M],current_curv_box,sizeof(JPXT_CURV));

                        xdpchnk.rd_e_param=e;

                        current_curv++;
                        off+=sizeof(JPXT_CURV);
                    }
                    else if (!memcmp((char *)&current_curv_box->TBox,"FTRX",4))
                    {
                        JPXT_FP_LIN_TRAFO *current_lin_trafo=(JPXT_FP_LIN_TRAFO*)(payload+off);
                        int M;
                        int T;

                        if (current_ftrx>=2)
                        {
                            free(ldr_image_mem.data);
                            printf("Wrong jpeg-xt profile!\n");
                            return 0;
                        }
                        M=current_lin_trafo->Mt & 0x0f;
                        T=(current_lin_trafo->Mt & 0xf0)>>4;
                        xdpchnk.colorspace=CS_XYZ;
                        XDP_SWAP_SHORT32(xt_lin_trafo[current_ftrx].LBox,current_lin_trafo->LBox);
                        xt_lin_trafo[current_ftrx].Mt=current_lin_trafo->Mt;
                        xt_lin_trafo[current_ftrx].a11=swap_float(current_lin_trafo->a11);
                        xt_lin_trafo[current_ftrx].a12=swap_float(current_lin_trafo->a12);
                        xt_lin_trafo[current_ftrx].a13=swap_float(current_lin_trafo->a13);
                        xt_lin_trafo[current_ftrx].a21=swap_float(current_lin_trafo->a21);
                        xt_lin_trafo[current_ftrx].a22=swap_float(current_lin_trafo->a22);
                        xt_lin_trafo[current_ftrx].a23=swap_float(current_lin_trafo->a23);
                        xt_lin_trafo[current_ftrx].a31=swap_float(current_lin_trafo->a31);
                        xt_lin_trafo[current_ftrx].a32=swap_float(current_lin_trafo->a32);
                        xt_lin_trafo[current_ftrx].a33=swap_float(current_lin_trafo->a33);
                        if (M==7 && T==0)
                        {
                            xdpchnk.base_matrix[0]=xt_lin_trafo[current_ftrx].a11;
                            xdpchnk.base_matrix[1]=xt_lin_trafo[current_ftrx].a12;
                            xdpchnk.base_matrix[2]=xt_lin_trafo[current_ftrx].a13;
                            xdpchnk.base_matrix[3]=xt_lin_trafo[current_ftrx].a21;
                            xdpchnk.base_matrix[4]=xt_lin_trafo[current_ftrx].a22;
                            xdpchnk.base_matrix[5]=xt_lin_trafo[current_ftrx].a23;
                            xdpchnk.base_matrix[6]=xt_lin_trafo[current_ftrx].a31;
                            xdpchnk.base_matrix[7]=xt_lin_trafo[current_ftrx].a32;
                            xdpchnk.base_matrix[8]=xt_lin_trafo[current_ftrx].a33;
                        }
                        if (M==8 && T==0)
                        {
                            xdpchnk.resi_matrix[0]=xt_lin_trafo[current_ftrx].a11;
                            xdpchnk.resi_matrix[1]=xt_lin_trafo[current_ftrx].a12;
                            xdpchnk.resi_matrix[2]=xt_lin_trafo[current_ftrx].a13;
                            xdpchnk.resi_matrix[3]=xt_lin_trafo[current_ftrx].a21;
                            xdpchnk.resi_matrix[4]=xt_lin_trafo[current_ftrx].a22;
                            xdpchnk.resi_matrix[5]=xt_lin_trafo[current_ftrx].a23;
                            xdpchnk.resi_matrix[6]=xt_lin_trafo[current_ftrx].a31;
                            xdpchnk.resi_matrix[7]=xt_lin_trafo[current_ftrx].a32;
                            xdpchnk.resi_matrix[8]=xt_lin_trafo[current_ftrx].a33;
                            xdpchnk.resi_identity=0;
                        }
                        current_ftrx++;
                        off+=sizeof(JPXT_FP_LIN_TRAFO);
                    }
                    else if (!memcmp((char *)&current_curv_box->TBox,"LPTS",4))
                    {
                        JPXT_BASE_NON_LINEAR_TRAFO *current_trafo=(JPXT_BASE_NON_LINEAR_TRAFO*)(payload+off);
                        lpts_td0=(current_trafo->td0td1 & 0xF0) >> 4;
                        lpts_td1=current_trafo->td0td1 & 0x0F;
                        lpts_td2=current_trafo->td2td3 & 0x0F;
                        lpts_found=1;
                        off+=sizeof(JPXT_BASE_NON_LINEAR_TRAFO);
                    }
                    else if (!memcmp((char *)&current_curv_box->TBox,"DPTS",4))
                    {
                        JPXT_BASE_NON_LINEAR_TRAFO *current_trafo=(JPXT_BASE_NON_LINEAR_TRAFO*)(payload+off);
                        dpts_td0=(current_trafo->td0td1 & 0xF0) >> 4;
                        dpts_td1=current_trafo->td0td1 & 0x0F;
                        dpts_td2=(current_trafo->td2td3 & 0xF0) >> 4;
                        dpts_found=1;

                        off+=sizeof(JPXT_BASE_NON_LINEAR_TRAFO);
                    }
                    else if (!memcmp((char *)&current_curv_box->TBox,"CPTS",4))
                    {
                        JPXT_BASE_NON_LINEAR_TRAFO *current_trafo=(JPXT_BASE_NON_LINEAR_TRAFO*)(payload+off);
                        cpts_td0=(current_trafo->td0td1 & 0xF0) >> 4;
                        cpts_td1=current_trafo->td0td1 & 0x0F;
                        cpts_td2=(current_trafo->td2td3 & 0xF0) >> 4;
                        cpts_found=1;

                        off+=sizeof(JPXT_BASE_NON_LINEAR_TRAFO);
                    }
                    else if (!memcmp((char *)&current_curv_box->TBox,"RPTS",4))
                    {
                        JPXT_BASE_NON_LINEAR_TRAFO *current_trafo=(JPXT_BASE_NON_LINEAR_TRAFO*)(payload+off);
                        rpts_td0=(current_trafo->td0td1 & 0xF0) >> 4;
                        rpts_td1=current_trafo->td0td1 & 0x0F;
                        rpts_td2=(current_trafo->td2td3 & 0xF0) >> 4;
                        rpts_found=1;
                        off+=sizeof(JPXT_BASE_NON_LINEAR_TRAFO);
                    }
                    else if (!memcmp((char *)&current_curv_box->TBox,"OCON",4))
                    {
                        JPXT_OUT_CONV_BOX *current_convbox=(JPXT_OUT_CONV_BOX*)(payload+off);
                        ocon_to0=(current_convbox->to0to1 & 0xF0) >> 4;
                        ocon_to1=current_convbox->to0to1 & 0x0F;
                        ocon_to2=(current_convbox->to2to3 & 0xF0) >> 4;
                        ocon_found=1;
                        xdpchnk.Rb=(current_convbox->RbLfOcCeOl & 0x80) >> 4;
                        off+=sizeof(JPXT_OUT_CONV_BOX);
                    }
                    else
                        off+=lbox;

                } while (off<payload_size);
                if (current_curv<5 )
                {
                    free(ldr_image_mem.data);
                    printf("Wrong jpeg-xt profile!\n");
                    return XT_ERROR_INVALID_INFO;
                }
                if (ocon_found==0 || rpts_found==0 || cpts_found==0 || dpts_found==0 || lpts_found==0)
                {
                    printf("Invalid file format!\nOperation aborted.\n");
                    free(ldr_image_mem.data);
                    free_memory_image(hdr_image_mem);
                    return 0;
                }
                xdpchnk.expval=swap_float(xtcurvs[ocon_to0].p3);

                xdpchnk.log_params_hdr[XT_P1]=swap_float(xtcurvs[rpts_td0].p1);
                xdpchnk.log_params_hdr[XT_P2]=swap_float(xtcurvs[rpts_td0].p2);
                xdpchnk.log_params_hdr[XT_P3]=swap_float(xtcurvs[rpts_td0].p3);
                xdpchnk.log_params_hdr[XT_P4]=swap_float(xtcurvs[rpts_td0].p4);

                xdpchnk.log_params_ldr[XT_P1]=swap_float(xtcurvs[cpts_td0].p1);
                xdpchnk.log_params_ldr[XT_P2]=swap_float(xtcurvs[cpts_td0].p2);
                xdpchnk.log_params_ldr[XT_P3]=swap_float(xtcurvs[cpts_td0].p3);
                xdpchnk.log_params_ldr[XT_P4]=swap_float(xtcurvs[cpts_td0].p4);


                xdpchnk.min_r=swap_float(xtcurvs[dpts_td0].p1);
                xdpchnk.max_r=swap_float(xtcurvs[dpts_td0].p2);

                xdpchnk.min_g=swap_float(xtcurvs[dpts_td1].p1);
                xdpchnk.max_g=swap_float(xtcurvs[dpts_td1].p2);

                xdpchnk.min_b=swap_float(xtcurvs[dpts_td2].p1);
                xdpchnk.max_b=swap_float(xtcurvs[dpts_td2].p2);

                xdpchnk.hdr_gamma=swap_float(xtcurvs[dpts_td0].p3);

                xdpchnk.p1=swap_float(xtcurvs[lpts_td0].p1);
                xdpchnk.p2=swap_float(xtcurvs[lpts_td0].p2);
                xdpchnk.p3=swap_float(xtcurvs[lpts_td0].p3);
                xdpchnk.epsilon_div=swap_float(xtcurvs[rpts_td0].p3);
                xdpchnk.ldr_gamma=swap_float(xtcurvs[lpts_td0].p2);

                if (xdpchnk.colorspace==CS_XYZ)
                {
                    if (current_ftrx<1)
                    {
                        free(ldr_image_mem.data);
                        printf("Wrong jpeg-xt profile!\n");
                        return XT_ERROR_INVALID_INFO;
                    }
                }
            }
            else if (!memcmp((char *)&xt_box_header->TBox,"RESI",4))
            {
                unsigned short chsz;
				unsigned short sz;
                XDP_SWAP_SHORT16(chsz,xt_box_header->Le);
                sz=chsz-sizeof(JPXT_SUPERBOX)+2;
                memcpy(hdr_image_mem->data+resi_off,payload,sz);
                hdr_image_mem->data_size+=sz;
                resi_off+=sz;

            }
        }
    } while(all_box_ok==0);
    ldr_image=jpeg_load_mem(&ldr_image_mem);
    if (ldr_image==NULL)
    {

        return 0;
    }



    free(ldr_image_mem.data);

    if (g_log_decode==1)
        printf("Use LOG decoding.\n");
    if (xdpchnk.p1>0)
        printf("Linearize LDR with SRGB profile.\n");
    if (xdpchnk.colorspace==CS_XYZ)
        printf("Use XYZ transformation.\n");
    if (xdpchnk.rd_e_param==0)
        printf("Centered rounding mode enabled.\n");
    if (xdpchnk.colorspace==CS_XYZ)
    {
        if (xdpchnk.resi_identity==1)
            printf("Do not use XYZ transformation on residual data.\n");
    }
    printf("LDR gamma : %f\n",xdpchnk.ldr_gamma);
    printf("HDR gamma : %f\n",xdpchnk.hdr_gamma);
    if (grayscale==0)
    {
        printf("max values : %f, %f, %f\n",xdpchnk.max_r,xdpchnk.max_g,xdpchnk.max_b);
        printf("min values : %f, %f, %f\n",xdpchnk.min_r,xdpchnk.min_g,xdpchnk.min_b);
    }
    else
    {
        printf("max values : %f\n",xdpchnk.max_r);
        printf("min values : %f\n",xdpchnk.min_r);
    }
    // Call XDP_ExpandHDR
    output_image=XDP_ExpandHDR(&xdpchnk,ldr_image,hdr_image_mem);
    if (output_image==NULL)
    {
        free_image(ldr_image);
        free_memory_image(hdr_image_mem);
        return 0;
    }
    if (!strcmp(get_extension(out_filename),"pfm"))
            pfm_write(output_image,out_filename);
    if (!strcmp(get_extension(out_filename),"hdr") || !strcmp(get_extension(out_filename),"rgbe"))
        // Write float buffer in radiance format
        hdr_write(output_image,out_filename);
    free_image(ldr_image);
    free_memory_image(hdr_image_mem);
    return 1;
}

